/*
 * Created on 28-mrt-2008
 */
package be.ASPL.presentationlayer;

import java.util.*;
import java.util.Map.Entry;

import be.SIRAPRISE.typeimplementations.*;

/**
 * Utility class to produce ASPL list values from a SIRA_PRISE query response
 * 
 * @author Erwin Smout
 */
public class ASPLListFactory {

	/**
	 * Creates a ListValue off the textual result of a SIRA_PRISE query
	 * 
	 * @param listFieldName
	 *            The name of the dynamic-list field on the presentation
	 * @param relation
	 *            the input relation to be turned into a presentation list
	 * @param locale
	 *            the locale to use
	 * @return A list value with columns as determined by the text value
	 * @throws ASPLException
	 *             if the function failed
	 * @Deprecated - replace with {@link #varColumnListFromRelation(NameIdentifier, Relation, Locale)}
	 */
	public static DynamicColumnListValue varColumnListFromRelation (String listFieldName, Relation relation, Locale locale) throws ASPLException {
		return varColumnListFromRelation(NameIdentifier.get(listFieldName), relation, locale);
	}

	/**
	 * Creates a ListValue off the textual result of a SIRA_PRISE query
	 * 
	 * @param listFieldName
	 *            The name of the dynamic-list field on the presentation
	 * @param relation
	 *            the input relation to be turned into a presentation list
	 * @param locale
	 *            the locale to use
	 * @return A list value with columns as determined by the text value
	 * @throws ASPLException
	 *             if the function failed
	 */
	public static DynamicColumnListValue varColumnListFromRelation (NameIdentifier listFieldName, Relation relation, Locale locale) throws ASPLException {
		DynamicColumnListValue listValue = new DynamicColumnListValue(listFieldName.getString());

		Heading heading = relation.getHeading();
		for (Entry<NameIdentifier, TypeDeclaration> me : heading.getEntriesWithIdentifier()) {
			listValue.addColumn(me.getKey().getString());
		}

		for (Tuple tuple : relation) {
			HashMap<String, PresentationValue> m = new HashMap<String, PresentationValue>();
			for (Entry<NameIdentifier, TypeDeclaration> me : tuple.getHeading().getEntriesWithIdentifier()) {
				final NameIdentifier attributeName = me.getKey();
				final TypeDeclaration attributeTypeDeclaration = me.getValue();

				if (attributeTypeDeclaration.isScalar()) {
					m.put(attributeName.getString(), new StringValue(tuple.valueText(attributeName)));
				} else {
					Heading nonScalarAttributeHeadingType = ((NonScalarTypeDeclaration) attributeTypeDeclaration).getHeading();
					Relation attributeRelationValue;
					if (attributeTypeDeclaration.getTypeNameIdentifier().getString().equalsIgnoreCase(TYPENAMES.TUPLE)) {
						Tuple attributeTupleValue = tuple.tvaValue(attributeName);
						attributeRelationValue = new Relation(nonScalarAttributeHeadingType, new RelationBuffer(Arrays.asList(new TupleBuffer[] { attributeTupleValue.getTupleBuffer() })));
					} else {
						attributeRelationValue = tuple.rvaValue(attributeName);
					}
					m.put(attributeName.getString(), varColumnListFromRelation(attributeName, attributeRelationValue, locale));
				}
			}
			listValue.append(m);
		}

		return listValue;
	}

	/**
	 * @param listField
	 *            the listfield with its column definitions
	 * @param r
	 *            the relation to be turned into a presentation list
	 * @param userData
	 *            the userData object
	 * @param locale
	 *            the locale to use
	 * @return A ListValue holding all the tuples of the given relation as a row and all the attributes of those tuples as a column (if they also appear as a column in the given listfield)
	 * @throws ASPLException
	 *             if the function failed
	 */
	public static final ListValue getASPLListValue (PresentationField listField, Relation r, UserData userData, Locale locale) throws ASPLException {
		ListValue listValue = new ListValue(listField.getName());

		for (Tuple tuple : r) {
			HashMap<String, PresentationValue> m = new HashMap<String, PresentationValue>();
			for (Entry<NameIdentifier, TypeDeclaration> me : tuple.getHeading().getEntriesWithIdentifier()) {
				final NameIdentifier attributeName = me.getKey();
				final PresentationField listColumnField = listField.getColumn(attributeName.getString());
				if (listColumnField != null) {
					m.put(attributeName.getString(), (listColumnField.getType() instanceof List ? getASPLListValue(listColumnField, tuple.rvaValue(attributeName), userData, locale) : listColumnField.getType().encode(tuple.valueText(attributeName), userData, locale)));
				}
			}
			listValue.append(m);
		}
		return listValue;
	}
}