package be.ASPL.presentationlayer;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import be.SIRAPRISE.util.ISODateFormat;
import be.SIRAPRISE.util.MyMessageFormat;

/**
 * GenericDate is the presentation type for date fields.
 * 
 * @author Erwin
 */
public class DateType extends PresentationType<DateValue> {

	/**
	 * 
	 */
	private static final long MILLISECONDSINDAY = 1000 * 60 * 60 * 24;

	/**
	 * @param date
	 *            -
	 * @return -
	 */
	public static DateValue fromDate (Date date) {
		return new DateValue(date);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#checkValidWaarValue(be.ASPL.presentationlayer.WaarValue, be.ASPL.presentationlayer.UserData)
	 */
	@Override
	public DateValue checkValidWaarValue (PresentationValue value, UserData userData) throws InvalidAttributeValueException {
		if (!(value instanceof DateValue)) {
			Locale locale = userData.getLocale();
			throw new InvalidAttributeValueException(value.toASPLString(userData), this.getClass().getSimpleName(), MyMessageFormat.format(I18N.getString(locale, "DynamicColumnList.InvalidValueClass"), new String[] { getClass().getName(), value.getClass().getName() })); //$NON-NLS-1$
		}
		return (DateValue) value;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#encode(byte[], be.ASPL.presentationlayer.UserData, java.util.Locale)
	 */
	@Override
	public DateValue encode (byte[] partBytes, UserData userData, Locale locale) {
		throw new UnsupportedOperationException();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.waar.AttributeType#validate(java.lang.String)
	 */
	public DateValue encode (String val, UserData userData, Locale locale) throws InvalidAttributeValueException {
		if (val.length() == 0) {
			return null;
		} else {
			Date d;
			try {
				SimpleDateFormat preferredDateFormat = userData == null ? ISODateFormat.getISODateFormatInstance() : userData.getPreferredDateFormat();
				d = preferredDateFormat.parse(val);
			} catch (ParseException e1) {
				throw new InvalidAttributeValueException(val, this.getClass().getSimpleName(), e1.getMessage());
			}
			return fromDate(d);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.PresentationType#getDefaultPresentationRenderingStyle()
	 */
	@Override
	public Renderer getDefaultPresentationRenderingStyle ( ) {
		return TextBoxRenderer.getInstance();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.PresentationAttributeType#decode(be.erwinsmout.ASPL.WaarValue)
	 */
	public String getNakedStringRepresentation (PresentationValue presentationValue, OnLinePresentationFunction userFunction, PresentationField presentationField, Locale locale) {
		UserData userData = userFunction.getUserData();
		SimpleDateFormat preferredDateFormat = userData == null ? ISODateFormat.getISODateFormatInstance() : userData.getPreferredDateFormat();
		String result = preferredDateFormat.format(((DateValue) presentationValue).getDate());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.PresentationType#getPresentationRenderingStyle(java.lang.String)
	 */
	@Override
	public Renderer getPresentationRenderingStyle (String renderingStyleName) {
		if (renderingStyleName.equalsIgnoreCase("PASSWORD")) { //$NON-NLS-1$
			return PassWordRenderer.getInstance();
		}
		return TextBoxRenderer.getInstance();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#getWaarValue(boolean, java.util.Locale)
	 */
	@Override
	public PresentationValue getWaarValue (boolean b, Locale locale) {
		throw new UnsupportedOperationException();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#getWaarValue(int)
	 */
	@Override
	public PresentationValue getWaarValue (int v, Locale locale) {
		return fromDate(new Date(v * MILLISECONDSINDAY));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#getWaarValue(long)
	 */
	@Override
	public PresentationValue getWaarValue (long v, Locale locale) {
		return new DateValue(new Date(v - (v % MILLISECONDSINDAY)));
	}
}
