package be.ASPL.presentationlayer;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.Date;

/**
 * GenericDateValue is the presentation layer value Class for date values
 * 
 * @author Erwin
 */
public final class DateValue implements PresentationValue, Externalizable {

	/**
	 * The date value
	 */
	private Date date;

	/**
	 * Constructor usable for serialization/externalization processes. Do not use in other circumstances, as there is no way to set the Date value afterwards.
	 */
	public DateValue ( ) {

	}

	/**
	 * Creates the presentation layer value
	 * 
	 * @param date
	 *            The date value
	 */
	public DateValue (Date date) {
		this.date = date;
	}

	/**
	 * Gets The date value
	 * 
	 * @return The date value
	 */
	public final Date getDate ( ) {
		return date;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.io.Externalizable#readExternal(java.io.ObjectInput)
	 */
	@Override
	public final void readExternal (ObjectInput in) throws IOException {
		this.date = new Date(in.readLong());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public final String toString ( ) {
		return date.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.WaarValue#toWAARString(be.ASPL.presentationlayer.UserData)
	 */
	@Override
	public final String toASPLString (UserData userData) {
		return userData.getPreferredDateFormat().format(date);
	}

	/**
	 * @serialData the date value is written as a java long
	 */
	@Override
	public final void writeExternal (ObjectOutput out) throws IOException {
		out.writeLong(date.getTime());
	}
}
