package be.ASPL.presentationlayer;

import java.io.*;
import java.lang.reflect.InvocationTargetException;
import java.util.*;

import javax.servlet.annotation.*;
import javax.servlet.http.*;

import be.SIRAPRISE.util.Localizer;
import be.SIRAPRISE.util.MyReadOnlySet;

/**
 * @author Erwin
 */
public abstract class OnLinePresentationFunction extends PresentationFunction {

	/**
	 * The calling Servlet
	 */
	private HttpServlet callingServlet;

	/**
	 * The map where user context can be saved
	 */
	private HashMap<String, Object> functionUserContext = new HashMap<String, Object>();

	/**
	 * 
	 */
	private boolean hasHelpPresentation;

	/**
	 * flag indicating that the user function has the ASPL system help presentation "popped up"
	 */
	private boolean helpPresentationPoppedUp;

	/**
	 * Comment for <code>htmlHandler</code>
	 */
	private HTMLPresentationHandler htmlHandler;

	/**
	 * 
	 */
	private String memorizeData;

	/**
	 * 4 distinc Nestinglevels,each with their own nextrowstyleclassname, are supported
	 */
	private int nextRowStyleClassNameIndex[] = new int[] { 0, 0, 0, 0 };

	/**
	 * The function's current Presentation
	 */
	private Presentation presentation;

	/**
	 * Comment for <code>presentationStack</code>
	 */
	private LinkedList<Presentation> presentationStack = new LinkedList<Presentation>();

	/**
	 * Comment for <code>queryString</code>
	 */
	private String queryString;

	/**
	 * Comment for <code>returnParms</code>
	 */
	private String returnParms;

	/**
	 * Comment for <code>returnPath</code>
	 */
	private String returnPath;

	/**
	 * 
	 */
	private String[] rowStyleClassNames = new String[] { "tr1", "tr2" }; //$NON-NLS-1$//$NON-NLS-2$

	/**
	 * Comment for <code>servletPath</code>
	 */
	private String servletPath;

	/**
	 * Comment for <code>suspended</code>
	 */
	private boolean suspended = false;

	/**
	 * Creates the presentation function
	 */
	public OnLinePresentationFunction ( ) {
		try {
			Architecture.getResourceReader(this.getClass().getSimpleName() + "Help" + Presentation.PRESENTATIONSUFFIX, this.getClass()); //$NON-NLS-1$
			hasHelpPresentation = true;
		} catch (ResourceNotFoundException e2) {
			hasHelpPresentation = (getClass().getAnnotation(HelpPresentation.class) != null);
		}
	}

	/**
	 * Finalizes the execution of the presentation logic
	 */
	final void endPresentationLogic ( ) {
		// if (getDbLink() != null) {
		// getDbLink().close();
		// }
	}

	/**
	 * Fills in all values that appear in the request's parameters
	 * 
	 * @param requestParameters
	 *            The HTTP request from which the presentation values have to be obtained
	 * @param requestMultiPartConfigParts
	 * @throws InvalidAttributeValueException
	 *             If a textual representation of a value is not valid for the type of field it is assigned to.
	 * @throws MandatoryFieldMissingException
	 * @throws PartSizeExceedsLimitsException
	 * @throws MultiPartIOException
	 * @throws IncompletePartException
	 */
	void fillInPresentationValuesIgnoringDefaults (Map<String, String> requestParameters, Collection<Part> requestMultiPartConfigParts) throws InvalidAttributeValueException, MandatoryFieldMissingException, PartSizeExceedsLimitsException, MultiPartIOException, IncompletePartException {
		try {
			presentation.setPresentationValuesIgnoringDefaults(requestParameters, requestMultiPartConfigParts, getUserData());
		} catch (IOException e) {
			throw new MultiPartIOException(e);
		}
	}

	/**
	 * @param requestParameters
	 * @param requestMultiPartConfigParts
	 * @throws InvalidAttributeValueException
	 * @throws PartSizeExceedsLimitsException
	 * @throws MultiPartIOException
	 * @throws IncompletePartException
	 */
	void fillInPresentationValuesIncludingDefaults (Map<String, String> requestParameters, Collection<Part> requestMultiPartConfigParts) throws InvalidAttributeValueException, PartSizeExceedsLimitsException, MultiPartIOException, IncompletePartException {
		final Properties locallyConfiguredPresentationDefaults = new Properties();
		try {
			try (FileInputStream fis = new FileInputStream(System.getProperty("user.home") + File.separatorChar + getApplicationName() + File.separatorChar + presentation.getName() + ".defaults.properties");) {//$NON-NLS-1$//$NON-NLS-2$
				locallyConfiguredPresentationDefaults.load(fis);
			}
		} catch (FileNotFoundException e1) {

		} catch (IOException e1) {
			log(e1);
		}
		try {
			presentation.setPresentationValuesIncludingDefaults(requestParameters, requestMultiPartConfigParts, getUserData(), locallyConfiguredPresentationDefaults);
		} catch (IOException e) {
			throw new MultiPartIOException(e);
		}
	}

	/**
	 * Gets The Servlet object that invoked this function.
	 * 
	 * @return The Servlet object that invoked this function.
	 */
	HttpServlet getCallingServlet ( ) {
		return callingServlet;
	}

	/**
	 * @return
	 */
	final String getFormEncodingType ( ) {
		return (getCallingServlet().getClass().getAnnotation(MultipartConfig.class) == null) ? "application/x-www-form-urlencoded" : "multipart/form-data"; //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * @return the html output to be displayed
	 */
	String getHTMLOutput ( ) {
		return htmlHandler.createOutputData(getUserLocale());
	}

	/**
	 * @param tableNestingLevel
	 *            parameter indicating the nesting level of the table, thus indicating which subscript of rowstyle to use
	 * @return
	 */
	final String getNextRowStyleClassName (int tableNestingLevel) {
		String nextRowStyleClassName = rowStyleClassNames[nextRowStyleClassNameIndex[tableNestingLevel]++];
		if (nextRowStyleClassNameIndex[tableNestingLevel] >= rowStyleClassNames.length) {
			nextRowStyleClassNameIndex[tableNestingLevel] = 0;
		}
		return nextRowStyleClassName;
	}

	/**
	 * @return the querystring
	 */
	String getQueryString ( ) {
		return queryString;
	}

	/**
	 * @return the return parms
	 */
	String getReturnParms ( ) {
		return returnParms;
	}

	/**
	 * @return the return url
	 */
	String getReturnPath ( ) {
		return returnPath;
	}

	/**
	 * @return the servletpath
	 */
	String getServletPath ( ) {
		return servletPath;
	}

	/**
	 * @return
	 */
	final boolean hasHelpPresentation ( ) {
		return hasHelpPresentation;
	}

	/**
	 * @param applicationName
	 *            The application name
	 * @param queryString_p
	 *            The query String from the HttpServletRequest
	 * @param userData
	 *            The userData object holding all data regarding the user
	 * @param returnPath_p
	 *            The path to return to once this function is completed
	 * @param returnParms_p
	 *            The parms to give when returning
	 * @param functionName
	 *            The name of this function
	 * @param requestParameters
	 *            The HttpServletRequest object
	 * @param requestMultiPartConfigParts
	 * @param callingServlet_p
	 *            The invoking servlet (to which our log requests must be delegated)
	 * @param requestServletPath
	 */
	void initialize (String applicationName, String queryString_p, UserData userData, String returnPath_p, String returnParms_p, String functionName, Map<String, String> requestParameters, Collection<Part> requestMultiPartConfigParts, HttpServlet callingServlet_p, String requestServletPath) {
		setApplicationName(applicationName);
		this.servletPath = applicationName + requestServletPath;
		this.queryString = queryString_p;
		setUserData(userData);
		this.returnPath = returnPath_p;
		this.returnParms = returnParms_p;
		this.callingServlet = callingServlet_p;

		try {
			Class<? extends Presentation> initialPresentationClass = getInitialPresentationClass();
			if (initialPresentationClass != null) {
				setPresentation(initialPresentationClass);
			} else {
				String presentationName = getInitialPresentationName();
				if (presentationName == null) {
					presentationName = functionName;
				}
				setPresentation(presentationName);
			}

			try {
				fillInPresentationValuesIncludingDefaults(requestParameters, requestMultiPartConfigParts);
				doInitialLogic();
			} catch (ASPLException e) {
				// If the requested presentation could not be set for whatever reason, an "emergency escape" presentation is set instead,
				// so that a presentation message will not be set on a null presenetation
				setPresentationMessage(e);
			}
		} catch (PresentationNotSetException e1) {
			setPresentationMessage(e1);
		}
	}

	/**
	 * Gets helpPresentationPoppedUp
	 * 
	 * @return helpPresentationPoppedUp.
	 */
	final boolean isHelpPresentationPoppedUp ( ) {
		return helpPresentationPoppedUp;
	}

	/**
	 * Gets the suspended flag
	 * 
	 * @return the suspended flag
	 */
	boolean isSuspended ( ) {
		return suspended;
	}

	/**
	 * @param helpPresentationPoppedUp
	 */
	final void setHelpPresentationPoppedUp (boolean helpPresentationPoppedUp) {
		this.helpPresentationPoppedUp = helpPresentationPoppedUp;
	}

	/**
	 * Sets the presentation message text to the message text of the given WaarException, adapted to the user's national language
	 * 
	 * @param e
	 *            The WaarException of whose appearance the user is to be informed in his own language
	 */
	final void setPresentationMessage (ASPLException e) {
		// presentation.setMessage(e.getMessageText(getApplicableUserLanguage()));
		presentation.setMessage(e.getMessage(getUserLocale()));
	}

	/**
	 * Suspends this function until the newly invoked function returns
	 */
	void suspend ( ) {
		suspended = true;
	}

	/**
	 * 
	 */
	void unSuspend ( ) {
		suspended = false;
		try {
			doResumeLogic();
		} catch (ASPLException e) {
			setPresentationMessage(e);
		}
	}

	/**
	 * Gets the localizer for this function
	 * 
	 * @return the localizer for this function
	 */
	protected abstract Localizer getApplicationLocalizer ( );

	/**
	 * @return
	 */
	protected String getDescriptionLocalizationKey ( ) {
		OnLinePresentationFunctionCharacteristics a = this.getClass().getAnnotation(OnLinePresentationFunctionCharacteristics.class);
		return a == null ? null : a.descriptionLocalizationKey();
	}

	/**
	 * Gets the configured value for the named context parameter and the servlet through which this function was invoked
	 * 
	 * @param name
	 *            The name of the context parameter whose configured value is to be returned
	 * @return the configured value for the named context parameter and the servlet through which this function was invoked
	 */
	protected final String getInitParameter (String name) {
		return callingServlet.getInitParameter(name);
	}

	/**
	 * Gets the configured value for the named context parameter and the servlet through which this function was invoked
	 * 
	 * @param name
	 *            The name of the context parameter whose configured value is to be returned
	 * @param defaultValue
	 *            The value to be returned if no value is configured in the servlet context
	 * @return the configured value for the named context parameter and the servlet through which this function was invoked
	 */
	protected final String getInitParameter (String name, String defaultValue) {
		final String s = callingServlet.getInitParameter(name);
		return s == null ? defaultValue : s;
	}

	/**
	 * Gets the names of the configured context parameters for the servlet through which this function was invoked
	 * 
	 * @return the names of the configured context parameters for the servlet through which this function was invoked
	 */
	protected final Enumeration<String> getInitParameterNames ( ) {
		return callingServlet.getInitParameterNames();
	}

	/**
	 * @return -
	 */
	protected final MyReadOnlySet<String> getUserRoles ( ) {
		return getUserData().getUserRoles();
	}

	/**
	 * <p>
	 * Transfers control to another dialog. Return from that dialog will not pass through the transferring function again upon return. The dialog that is transferred to will be initialized just as if it was invoked by the user clicking a URL. This means in particular its doInitialLogic() method will be called.
	 * </p>
	 * <p>
	 * Note that while the method facilitates "transfer of control", this transfer of control is at the level of the user dialogs. The java code that issues this tranfer of control (invoked this method) will receive control again and continue processing. In most cases, it will be wise to call transferTo only as the last bit of processing in your code.
	 * </p>
	 * 
	 * @param transferToFunction
	 *            The function to transfer dialog control to
	 * @param functionParameters
	 *            A mapping of parameter names to parameter values that can be used by the initialization process to fill in the transferred-to dialog's presentation object.
	 */
	protected final void transferTo (OnLinePresentationFunction transferToFunction, Map<String, String> functionParameters) {
		final PresentationFunctionStack functionStack = getUserData().getUserSession().getFunctionStack();
		functionStack.removeLast(); // remove ourselves from the stack
		functionStack.addLast(transferToFunction); // add the passed-in function to the stack
		transferToFunction.initialize(getApplicationName(), getQueryString(), getUserData(), getReturnPath(), getReturnParms(), transferToFunction.getClass().getSimpleName(), functionParameters, new LinkedList<Part>(), getCallingServlet(), getServletPath());
	}

	/**
	 * <p>
	 * Transfers control to another dialog. Return from that dialog will pass through the transferring function again upon return. The dialog that is transferred to will be initialized just as if it was invoked by the user clicking a URL. This means in particular its doInitialLogic() method will be called.
	 * </p>
	 * <p>
	 * Note that while the method facilitates "transfer of control", this transfer of control is at the level of the user dialogs. The java code that issues this tranfer of control (invoked this method) will receive control again and continue processing. In most cases, it will be wise to call transferTo only as the last bit of processing in your code.
	 * </p>
	 * 
	 * @param invokeFunction
	 *            The function to transfer dialog control to
	 * @param functionParameters
	 *            A mapping of parameter names to parameter values that can be used by the initialization process to fill in the transferred-to dialog's presentation object.
	 */
	protected final void invoke (OnLinePresentationFunction invokeFunction, Map<String, String> functionParameters) {
		getUserData().getUserSession().getFunctionStack().addLast(invokeFunction); // add the passed-in function to the stack
		invokeFunction.initialize(getApplicationName(), getQueryString(), getUserData(), getReturnPath(), getReturnParms(), invokeFunction.getClass().getSimpleName(), functionParameters, new LinkedList<Part>(), getCallingServlet(), getServletPath());
		//FIXME one of the two initialize() calls in invoke/transferTo must be flawed.
	}

	/**
	 * Performs an action other than 'OK', 'HELP', 'CANCEL', 'MAIN' or 'LOGOFF'
	 * 
	 * @param action
	 *            the name of the requested action
	 * @throws ASPLException
	 *             -
	 */
	@SuppressWarnings("static-method")
	public void doActionLogic (String action) throws ASPLException {
		throw new ActionUnknownException(action);
	}

	/**
	 * Performs the 'cancel' action
	 */
	public void doCancel ( ) {

	}

	/**
	 * Function is called when the presentation is initialized, after the presentation object has been filled. The default implementation does nothing at all.
	 * 
	 * @throws ASPLException
	 *             -
	 */
	@SuppressWarnings("unused")
	public void doInitialLogic ( ) throws ASPLException {

	}

	/**
	 * Performs the presentation logic
	 * 
	 * @throws ASPLException
	 *             -
	 */
	public abstract void doPresentationLogic ( ) throws ASPLException;

	/**
	 * Performs the logic when the function is returned to from another invoked function
	 * 
	 * @throws ASPLException
	 *             -
	 */
	@SuppressWarnings("unused")
	public void doResumeLogic ( ) throws ASPLException {

	}

	/**
	 * Gets an object from the function context
	 * 
	 * @param name
	 *            The name under which the object was previuosly stored in the function context
	 * @return the object saved in the functioncontext under the given name, or null if no object was saved under the given name.
	 * @throws NotFoundException
	 *             If the function context does not contain the named element
	 */
	public final Object getFromFunctionContext (String name) throws NotFoundException {
		if (!functionUserContext.containsKey(name)) {
			throw new NotFoundException();
		}
		return functionUserContext.get(name);
	}

	/**
	 * Gets an object from the function context
	 * 
	 * @param name
	 *            The name under which the object was previuosly stored in the function context
	 * @return the object saved in the functioncontext under the given name, or null if no object was saved under the given name.
	 * @throws FunctionContextDoesNotHoldThisPropertyException
	 *             If the function context does not contain the named element
	 */
	public final Object getFromFunctionContextMandatory (String name) throws FunctionContextDoesNotHoldThisPropertyException {
		if (!functionUserContext.containsKey(name)) {
			throw new FunctionContextDoesNotHoldThisPropertyException(name);
		}
		return functionUserContext.get(name);
	}

	/**
	 * Retrieves an object from the user session context
	 * 
	 * @param name
	 *            The name under which something was saved in the session context
	 * @return The object saved under the given name
	 */
	public final Object getFromSessionContext (String name) {
		return getUserData().getFromSessionContext(name);
	}

	/**
	 * Gets the name of the initial presentation to be used with the function. The default implementation first checks if the class is annotated with an InitialPresentation annotation. If it is, then the Name property of that annotation is returned. If no such annotation is found, null is returned.
	 * 
	 * @return The name of the initial presentation to be used with the function. If null is returned, the system will use a presentation of the same name as the Business function (i.e. the servletname in http).
	 */
	public Class<? extends Presentation> getInitialPresentationClass ( ) {
		InitialPresentation a = this.getClass().getAnnotation(InitialPresentation.class);
		if (a == null) {
			return null;
		} else {
			return a.initialPresentationClass();
		}
	}

	/**
	 * Gets the name of the initial presentation to be used with the function. The default implementation first checks if the class is annotated with an InitialPresentation annotation. If it is, then the Name property of that annotation is returned. If no such annotation is found, null is returned.
	 * 
	 * @return The name of the initial presentation to be used with the function. If null is returned, the system will use a presentation of the same name as the Business function (i.e. the servletname in http).
	 */
	@SuppressWarnings("static-method")
	public String getInitialPresentationName ( ) {
		return null;
	}

	/**
	 * Gets the Cookie for this user Function. The cookie will be included in the response to the browser.
	 * 
	 * @return the Cookie for this user Function. The cookie will be included in the response to the browser.
	 */
	public final String getMemorizeData ( ) {
		return memorizeData;
	}

	/**
	 * Gets the current presentation
	 * 
	 * @return The current presentation
	 */
	public final Presentation getPresentation ( ) {
		return presentation;
	}

	/**
	 * Gets the Presentation Function Title to be displayed
	 * 
	 * @return the Presentation Function Title to be displayed
	 */
	public String getPresentationFunctionDescription ( ) {
		Localizer localizer;
		if ((localizer = getApplicationLocalizer()) != null) {
			String descriptionLocalizationKey;
			if ((descriptionLocalizationKey = getDescriptionLocalizationKey()) != null) {
				return localizer.getString(descriptionLocalizationKey, getUserLocale());
			}
		}

		return ""; //$NON-NLS-1$
	}

	/**
	 * Gets the value of the the named field on the presentation
	 * 
	 * @param presentationFieldName
	 *            The name of the presentation field whose value is to be retrieved
	 * @return The value of the field
	 */
	public final PresentationValue getPresentationValue (String presentationFieldName) {
		return presentation.getPresentationValues().get(presentationFieldName);
	}

	/**
	 * Checks whether the function has to retain the "focus" after the presentation logic has been performed, or whether it can be "left". The default implementation is to return the value coded in the LeaveFunctionOnExit annotation, or if there is none such, to return true. Override the method for any other behaviour.
	 * 
	 * @return true if the function is no longer to be in control after the processing logic has been performed, false if the function is to remain in control.
	 */
	public boolean leaveFunctionOnExit ( ) {
		OnLinePresentationFunctionCharacteristics a = this.getClass().getAnnotation(OnLinePresentationFunctionCharacteristics.class);
		return a == null ? true : a.leaveFunctionOnExit();
	}

	/**
	 * Method to log a message in the application server's log.
	 * 
	 * @param msg
	 *            The message to be logged by the application server
	 */
	public final void log (String msg) {
		callingServlet.log(msg);
	}

	/**
	 * Logs a throwable in the Application Server's log
	 * 
	 * @param e
	 *            The throwable to log
	 */
	public final void log (Throwable e) {
		callingServlet.log(e.getMessage(), e);
	}

	/**
	 * @throws NoStackedPresentationException
	 *             -
	 */
	public final void popoffPresentation ( ) throws NoStackedPresentationException {
		if (presentationStack.size() > 0) {
			Presentation revertToPresentation = presentationStack.removeLast();
			setPresentation(revertToPresentation);
		} else {
			throw new NoStackedPresentationException();
		}
	}

	/**
	 * Pops up another presentation, while retaining the current presentation on a stack
	 * 
	 * @param presentationClass
	 *            the Class of a Presentation to be instantiated as the presentation to pop up above the currently active presentation for this function
	 * @throws PresentationNotSetException
	 *             if the presentationClass cannot be instantiated using the Locale constructor
	 */
	public final void popupPresentation (Class<? extends Presentation> presentationClass) throws PresentationNotSetException {
		try {
			popupPresentation(presentationClass.newInstance());
		} catch (InstantiationException | IllegalAccessException | SecurityException | IllegalArgumentException e) {
			log(e);
			try {
				setPresentation(new PresentationProblemPresentation());
				setPresentationValue(PresentationProblemPresentation.EXCEPTIONCLASS, e.getClass().getName());
				setPresentationValue(PresentationProblemPresentation.EXCEPTIONMESSAGE, e.getMessage());
			} catch (ASPLException e1) {
				throw new RuntimeException("Neither the application's requested presentation, nor the architecture's presentation reporting this problem, could be set.  Problem with application's presentation was : " + e.getMessage() + ".  Problem with architecture's presentation was : " + e1.getMessage(), e); //$NON-NLS-1$ //$NON-NLS-2$
			}
			throw new PresentationNotSetException(presentationClass.getSimpleName(), getApplicationName(), e.getMessage());
		}
	}

	/**
	 * Pops up another presentation, while retaining the current presentation on a stack
	 * 
	 * @param popupPresentation
	 *            The new presentation to pop "above" the current
	 * @Deprecated - replace with the method that takes a Presentation object for its argument
	 */
	public final void popupPresentation (Presentation popupPresentation) {
		presentationStack.addLast(presentation);
		setPresentation(popupPresentation);
	}

	/**
	 * Pops up another presentation, while retaining the current presentation on a stack
	 * 
	 * @param presentationName
	 *            The name of the new presentation to pop "above" the current
	 * @throws PresentationNotSetException
	 *             -+
	 * @deprecated - replace with the method that takes a Presentation object for its argument
	 */
	@Deprecated
	public final void popupPresentation (String presentationName) throws PresentationNotSetException {
		presentationStack.addLast(presentation);
		setPresentation(presentationName);
	}

	/**
	 * Stores an object under the given name in the function context for later retrieval
	 * 
	 * @param name
	 *            The name under which the object is to be stored in the function context
	 * @param object
	 *            The object to be stored in the function context
	 */
	public final void saveInFunctionContext (String name, Object object) {
		functionUserContext.put(name, object);
	}

	/**
	 * Saves an object in the user session context
	 * 
	 * @param name
	 *            The name under which something should be saved in the context
	 * @param o
	 *            The object to save under the given name
	 */
	public final void saveInSessionContext (String name, Externalizable o) {
		getUserData().saveInSessionContext(name, o);
	}

	/**
	 * Sets the Cookie for this user Function. The cookie will be included in the response to the browser.
	 * 
	 * @param memorizeData
	 *            The value to be recorded in the Cookie.
	 */
	public final void setMemorizeData (String memorizeData) {
		this.memorizeData = memorizeData;
	}

	/**
	 * Sets the presentation currently to be associated with this presentation function
	 * 
	 * @param presentationClass
	 *            The name of the presentation to be set current
	 * @throws PresentationNotSetException
	 *             -
	 */
	public final void setPresentation (Class<? extends Presentation> presentationClass) throws PresentationNotSetException {
		Locale locale = getUserLocale();
		try {
			setPresentation(presentationClass.newInstance());
		} catch (InstantiationException | IllegalAccessException | SecurityException | IllegalArgumentException e0) {
			try {
				setPresentation(presentationClass.getConstructor(Locale.class).newInstance(locale));
			} catch (InstantiationException | IllegalAccessException | NoSuchMethodException | SecurityException | IllegalArgumentException | InvocationTargetException e) {
				log(e);
				try {
					setPresentation(new PresentationProblemPresentation());
					setPresentationValue(PresentationProblemPresentation.EXCEPTIONCLASS, e.getClass().getName());
					setPresentationValue(PresentationProblemPresentation.EXCEPTIONMESSAGE, e.getMessage() == null ? "-" : e.getMessage()); //$NON-NLS-1$
				} catch (ASPLException e1) {
					throw new RuntimeException("Neither the application's requested presentation, nor the architecture's presentation reporting this problem, could be set.  Problem with application's presentation was : " + e.getMessage() + ".  Problem with architecture's presentation was : " + e1.getMessage(), e); //$NON-NLS-1$ //$NON-NLS-2$
				}
				throw new PresentationNotSetException(presentationClass.getSimpleName(), getApplicationName(), e.getMessage());
			}
		}
	}

	/**
	 * Sets the current presentation for this function
	 * 
	 * @param presentation
	 *            The presentation to be shown to the user when the function is done processing
	 */
	public final void setPresentation (Presentation presentation) {
		this.presentation = presentation;
		htmlHandler = new HTMLPresentationHandler(this);
	}

	/**
	 * Sets the presentation currently to be associated with this presentation function
	 * 
	 * @param name
	 *            The name of the presentation to be set current
	 * @throws PresentationNotSetException
	 *             -
	 * @deprecated - prefer the method that takes a Presentation object
	 */
	@Deprecated
	public final void setPresentation (String name) throws PresentationNotSetException {
		Locale locale = getUserLocale();
		try {
			setPresentation(new Presentation(this.getClass(), name, locale));
		} catch (ASPLException e) {
			log(e);
			try {
				setPresentation(new PresentationProblemPresentation());
				setPresentationValue(PresentationProblemPresentation.EXCEPTIONCLASS, e.getClass().getName());
				setPresentationValue(PresentationProblemPresentation.EXCEPTIONMESSAGE, e.getMessage() == null ? "" : e.getMessage()); //$NON-NLS-1$
			} catch (ASPLException e1) {
				throw new RuntimeException("Neither the application's requested presentation, nor the architecture's presentation reporting this problem, could be set.  Problem with application's presentation was : " + e.getMessage() + ".  Problem with architecture's presentation was : " + e1.getMessage(), e); //$NON-NLS-1$ //$NON-NLS-2$
			}
			throw new PresentationNotSetException(name, getApplicationName(), e.getMessage());
		}
	}

	/**
	 * Sets the presentation message text in the language of the user.
	 * 
	 * @param localizedMessage
	 *            The localized message text to display to the user
	 */
	public final void setPresentationMessage (String localizedMessage) {
		presentation.setMessage(localizedMessage);
	}

	/**
	 * Sets a value on the presentation where that value is given in String format
	 * 
	 * @param presentationFieldName
	 *            -
	 * @param v
	 *            -
	 * @throws ASPLException
	 *             -
	 */
	public final void setPresentationValue (String presentationFieldName, boolean v) throws ASPLException {
		setPresentationValue(presentationFieldName, getPresentation().getPresentationField(presentationFieldName).getType().getWaarValue(v, getUserLocale()));
	}

	/**
	 * Sets a value on the presentation where that value is given in String format
	 * 
	 * @param presentationFieldName
	 *            -
	 * @param v
	 *            -
	 * @throws ASPLException
	 *             -
	 */
	public final void setPresentationValue (String presentationFieldName, char v) throws ASPLException {
		setPresentationValue(presentationFieldName, Character.toString(v));
	}

	/**
	 * Sets a value on the presentation where that value is given in String format
	 * 
	 * @param presentationFieldName
	 *            -
	 * @param v
	 *            -
	 * @throws ASPLException
	 *             -
	 */
	public final void setPresentationValue (String presentationFieldName, double v) throws ASPLException {
		setPresentationValue(presentationFieldName, Double.toString(v));
	}

	/**
	 * Sets a value on the presentation where that value is given in String format
	 * 
	 * @param presentationFieldName
	 *            -
	 * @param v
	 *            -
	 * @throws ASPLException
	 *             -
	 */
	public final void setPresentationValue (String presentationFieldName, float v) throws ASPLException {
		setPresentationValue(presentationFieldName, Float.toString(v));
	}

	/**
	 * Sets a value on the presentation where that value is given in String format
	 * 
	 * @param presentationFieldName
	 *            -
	 * @param v
	 *            -
	 * @throws ASPLException
	 *             -
	 */
	public final void setPresentationValue (String presentationFieldName, int v) throws ASPLException {
		setPresentationValue(presentationFieldName, getPresentation().getPresentationField(presentationFieldName).getType().getWaarValue(v, getUserLocale()));
	}

	/**
	 * Sets a value on the presentation where that value is given in String format
	 * 
	 * @param presentationFieldName
	 *            -
	 * @param v
	 *            -
	 * @throws ASPLException
	 *             -
	 */
	public final void setPresentationValue (String presentationFieldName, long v) throws ASPLException {
		setPresentationValue(presentationFieldName, getPresentation().getPresentationField(presentationFieldName).getType().getWaarValue(v, getUserLocale()));
	}

	/**
	 * Sets a value on the presentation (except if the presentation is the architecure's rescue presentation, which means that the presentation that was actually requested by the application could not be set in the first place, which will certainly prohibit to set any fields on them). This is a bit of a kludge. We should not even end up in the application anymore in that case.
	 * 
	 * @param presentationFieldName
	 *            -
	 * @param value
	 *            -
	 * @throws ASPLException
	 *             -
	 */
	public final void setPresentationValue (String presentationFieldName, PresentationValue value) throws ASPLException {
		PresentationValue checkedValue = presentation.getPresentationField(presentationFieldName).checkValidWaarValue(value, getUserData());
		// if (presentation.getPresentationFields().containsKey(presentationFieldName)) {
		presentation.getPresentationValues().put(presentationFieldName, checkedValue);
		// } else {
		// throw new PresentationFieldMissingException(presentationFieldName, presentation.getName(), getUserLocale());
		// }
	}

	/**
	 * Sets a value on the presentation where that value is given in String format
	 * 
	 * @param presentationFieldName
	 *            -
	 * @param v
	 *            -
	 * @throws ASPLException
	 *             -
	 */
	public final void setPresentationValue (String presentationFieldName, String v) throws ASPLException {
		presentation.getPresentationValues().put(presentationFieldName, presentation.getPresentationField(presentationFieldName).getType().encode(v, getUserData(), getUserLocale()));
	}
}