package be.ASPL.presentationlayer;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Locale;

import be.SIRAPRISE.util.ISOTimeFormat;
import be.SIRAPRISE.util.MyMessageFormat;

/**
 * GenericTime is the presentation type for time values. It encodes and decodes taking into account the user's preferred date and time formats.
 * 
 * @author Erwin
 */
public class TimeType extends PresentationType<TimeValue> {

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#checkValidWaarValue(be.ASPL.presentationlayer.WaarValue, be.ASPL.presentationlayer.UserData)
	 */
	@Override
	public TimeValue checkValidWaarValue (PresentationValue value, UserData userData) throws InvalidAttributeValueException {
		if (!(value instanceof TimeValue)) {
			Locale locale = userData.getLocale();
			throw new InvalidAttributeValueException(value.toASPLString(userData), this.getClass().getSimpleName(), MyMessageFormat.format(I18N.getString(locale, "DynamicColumnList.InvalidValueClass"), new String[] { getClass().getName(), value.getClass().getName() })); //$NON-NLS-1$
		}
		return (TimeValue) value;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#encode(byte[], be.ASPL.presentationlayer.UserData, java.util.Locale)
	 */
	@Override
	public TimeValue encode (byte[] partBytes, UserData userData, Locale locale) {
		throw new UnsupportedOperationException();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.waar.AttributeType#validate(java.lang.String)
	 */
	public TimeValue encode (String val, UserData userData, Locale locale) throws InvalidAttributeValueException {
		if (val.length() == 0) {
			return null;
		} else {
			try {
				return new TimeValue((userData == null ? ISOTimeFormat.getISOTimeFormatInstance() : userData.getPreferredTimeFormat()).parse(val));
			} catch (ParseException e1) {
				throw new InvalidAttributeValueException(val, this.getClass().getSimpleName(), e1.getMessage());
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.PresentationType#getDefaultPresentationRenderingStyle()
	 */
	@Override
	public Renderer getDefaultPresentationRenderingStyle ( ) {
		return TextBoxRenderer.getInstance();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.PresentationAttributeType#decode(be.erwinsmout.ASPL.WaarValue)
	 */
	public String getNakedStringRepresentation (PresentationValue presentationValue, OnLinePresentationFunction userFunction, PresentationField presentationField, Locale locale) {
		if (presentationValue instanceof TimeValue) {
			UserData userData = userFunction.getUserData();
			SimpleDateFormat userDateFormat = userData == null ? ISOTimeFormat.getISOTimeFormatInstance() : userData.getPreferredTimeFormat();
			return userDateFormat.format(((TimeValue) presentationValue).getDateTime());
		} else {
			throw new IllegalArgumentException(presentationValue.getClass().getName());
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.PresentationType#getPresentationRenderingStyle(java.lang.String)
	 */
	@Override
	public Renderer getPresentationRenderingStyle (String renderingStyleName) {
		if (renderingStyleName.equalsIgnoreCase("PASSWORD")) { //$NON-NLS-1$
			return PassWordRenderer.getInstance();
		}
		return TextBoxRenderer.getInstance();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#getWaarValue(boolean, java.util.Locale)
	 */
	@Override
	public PresentationValue getWaarValue (boolean b, Locale locale) {
		throw new UnsupportedOperationException();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#getWaarValue(int)
	 */
	@Override
	public PresentationValue getWaarValue (int v, Locale locale) {
		throw new UnsupportedOperationException();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#getWaarValue(long, java.util.Locale)
	 */
	@Override
	public PresentationValue getWaarValue (long v, Locale locale) {
		throw new UnsupportedOperationException();
	}
}
