/**
 * 
 */
package be.SIRAPRISE.birdwatcher;

import java.util.Set;

import be.ASPL.presentationlayer.ASPLApplicationException;
import be.ASPL.presentationlayer.DateValue;
import be.ASPL.presentationlayer.StringValue;
import be.SIRAPRISE.client.ConnectionClosedException;
import be.SIRAPRISE.client.DBConnection;
import be.SIRAPRISE.client.RelationValueSelectorFactory;
import be.SIRAPRISE.client.TransactionMode;
import be.SIRAPRISE.messages.ErrorMessageException;
import be.SIRAPRISE.typeimplementations.Relation;
import be.SIRAPRISE.typeimplementations.TYPENAMES;
import be.SIRAPRISE.util.ISODateFormat;
import be.SIRAPRISE.util.MyString;

/**
 * @author Erwin
 * 
 */
public abstract class BWWalksFunction extends BWWebClientFunction {

	/**
	 * @return
	 * @throws ASPLApplicationException
	 */
	final Relation getMatchingSpots ( ) throws ASPLApplicationException {

		String spotsQuery = "UNION(EXTEND(" + RELVARNAMES.SPOTS + ",(G(STRING( )))),TRANSFORM(" + RELVARNAMES.GSPOTS + ",(" + ATTRIBUTENAMES.SPOTCOUNT + "," + ATTRIBUTENAMES.WALKDATE + "," + ATTRIBUTENAMES.SPECIESNAME + ",G(THE_STRING(" + ATTRIBUTENAMES.GENDER + ")))))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$

		DateValue walkDate = (DateValue) getPresentationValue("WALKDATE"); //$NON-NLS-1$
		spotsQuery = "RESTRICT(" + spotsQuery + ",EQ(DATE(" + ISODateFormat.formatISO(walkDate.getDate()) + ")," + ATTRIBUTENAMES.WALKDATE + "))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$

		StringValue language = (StringValue) getPresentationValue(ATTRIBUTENAMES.LANGUAGE);
		if (language != null) {
			spotsQuery = "EXTEND(LEFTJOIN(" + spotsQuery + ",PROJECT(RESTRICT(" + RELVARNAMES.NLSPECIESNAME + ",EQ(" + ATTRIBUTENAMES.LANGUAGE + ",LANGUAGE(ISO2(" + language.getString() + ")))),(" + ATTRIBUTENAMES.SPECIESNAME + "," + ATTRIBUTENAMES.NLSPECIESNAME + ")),(" + ATTRIBUTENAMES.NLSPECIESNAME + "(STRING( )))),(NLANNOTATEDNAME(CONCAT(" + ATTRIBUTENAMES.SPECIESNAME + ",CONCAT(STRING( -),CONCAT(" + ATTRIBUTENAMES.NLSPECIESNAME + ",STRING(-)))))))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$//$NON-NLS-5$//$NON-NLS-6$//$NON-NLS-7$ //$NON-NLS-8$ //$NON-NLS-9$ //$NON-NLS-10$
		}

		
		try (DBConnection dbc = getDBConnection();) {
			return dbc.startTransaction(getUserData().getUserID(), true, "", new byte[] {}, TransactionMode.READONLY).execQueryAndEndTransaction(spotsQuery); //$NON-NLS-1$
		} catch (ConnectionClosedException e) {
			log(e);
			throw new ASPLApplicationException(e);
		} catch (ErrorMessageException e) {
			log(e);
			throw new ASPLApplicationException(e);
		}

	}

	/**
	 * @return
	 * @throws ASPLApplicationException
	 */
	final be.SIRAPRISE.typeimplementations.Relation getMatchingWalks ( ) throws ASPLApplicationException {

		String walkDatesQuery = RELVARNAMES.WALK;

		StringValue name = (StringValue) getPresentationValue(ATTRIBUTENAMES.SPECIESNAME);
		if (name != null) {
			Set<String> names = MyString.getNames(name.getString(), ',');
			String nameValueSelector = RelationValueSelectorFactory.getUnaryRelationValueSelectorExpressionText(names, ATTRIBUTENAMES.SPECIESNAME, TYPENAMES.STRING);
			walkDatesQuery = "PROJECT(JOIN(UNION(" + RELVARNAMES.SPOTS + ",PROJECT(" + RELVARNAMES.GSPOTS + ",(" + ATTRIBUTENAMES.SPOTCOUNT + "," + ATTRIBUTENAMES.WALKDATE + "," + ATTRIBUTENAMES.SPECIESNAME + ")))," + nameValueSelector + "),(" + ATTRIBUTENAMES.WALKDATE + "))"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$ //$NON-NLS-8$
		}

		DateValue walkDate = (DateValue) getPresentationValue("WALKDATE"); //$NON-NLS-1$
		if (walkDate != null) {
			walkDatesQuery = "RESTRICT(" + walkDatesQuery + ",EQ(DATE(" + ISODateFormat.formatISO(walkDate.getDate()) + ")," + ATTRIBUTENAMES.WALKDATE + "))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$
		} else {
			DateValue fromDate = (DateValue) getPresentationValue("FROMDATE"); //$NON-NLS-1$
			if (fromDate != null) {
				walkDatesQuery = "RESTRICT(" + walkDatesQuery + ",NOT(GT(DATE(" + ISODateFormat.formatISO(fromDate.getDate()) + ")," + ATTRIBUTENAMES.WALKDATE + ")))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$
			}

			DateValue toDate = (DateValue) getPresentationValue("TODATE"); //$NON-NLS-1$
			if (toDate != null) {
				walkDatesQuery = "RESTRICT(" + walkDatesQuery + ",GT(DATE(" + ISODateFormat.formatISO(toDate.getDate()) + ")," + ATTRIBUTENAMES.WALKDATE + "))"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$
			}
		}

		StringValue language = (StringValue) getPresentationValue(ATTRIBUTENAMES.LANGUAGE);
		if (language != null) {
			walkDatesQuery = "LEFTJOIN(" + walkDatesQuery + ",GROUP(TRANSFORM(LEFTJOIN(UNION(PROJECT(" + RELVARNAMES.SPOTS + ",(" + ATTRIBUTENAMES.WALKDATE + "," + ATTRIBUTENAMES.SPECIESNAME + ")),PROJECT(" + RELVARNAMES.GSPOTS + ",(" + ATTRIBUTENAMES.WALKDATE + "," + ATTRIBUTENAMES.SPECIESNAME + "))),PROJECT(RESTRICT(" + RELVARNAMES.NLSPECIESNAME + ",EQ(" + ATTRIBUTENAMES.LANGUAGE + ",LANGUAGE(ISO2(" + ((StringValue) getPresentationValue(ATTRIBUTENAMES.LANGUAGE)).getString() + ")))),(" + ATTRIBUTENAMES.SPECIESNAME + "," + ATTRIBUTENAMES.NLSPECIESNAME + "))," + ATTRIBUTENAMES.NLSPECIESNAME + "(STRING( ))),(" + ATTRIBUTENAMES.WALKDATE + "," + ATTRIBUTENAMES.SPECIESNAME + ",NLANNOTATEDNAME(CONCAT(" + ATTRIBUTENAMES.SPECIESNAME + ",CONCAT(STRING( -),CONCAT(" + ATTRIBUTENAMES.NLSPECIESNAME + ",STRING(-))))))),SPOTTEDSPECIESLIST(" + ATTRIBUTENAMES.SPECIESNAME + ",NLANNOTATEDNAME)),(SPOTTEDSPECIESLIST(RELATION(HEADING(" + ATTRIBUTENAMES.SPECIESNAME + "(STRING)NLANNOTATEDNAME(STRING))BODY())))),(" + ATTRIBUTENAMES.WALKDATE + ")"; //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$//$NON-NLS-5$//$NON-NLS-6$//$NON-NLS-7$ //$NON-NLS-8$ //$NON-NLS-9$ //$NON-NLS-10$ //$NON-NLS-11$ //$NON-NLS-12$ //$NON-NLS-13$ //$NON-NLS-14$ //$NON-NLS-15$ //$NON-NLS-16$ //$NON-NLS-17$ //$NON-NLS-18$ //$NON-NLS-19$ //$NON-NLS-20$ //$NON-NLS-21$
		}

		
		try (DBConnection dbc = getDBConnection();) {
			return dbc.startTransaction(getUserData().getUserID(), true, "", new byte[] {}, TransactionMode.READONLY).execQueryAndEndTransaction(walkDatesQuery); //$NON-NLS-1$
		} catch (ConnectionClosedException e) {
			log(e);
			throw new ASPLApplicationException(e);
		} catch (ErrorMessageException e) {
			log(e);
			throw new ASPLApplicationException(e);
		}
	}
}
