package be.SIRAPRISE.birdwatcher;

import java.util.*;

import be.ASPL.presentationlayer.*;
import be.SIRAPRISE.client.*;
import be.SIRAPRISE.client.jsba.*;
import be.SIRAPRISE.messages.ErrorMessageException;
import be.SIRAPRISE.typeimplementations.NameIdentifier;
import be.SIRAPRISE.typeimplementations.Relation;
import be.SIRAPRISE.typeimplementations.Tuple;

/**
 * The function to delete a new existing species
 * 
 * @author Erwin
 */
@InitialPresentation(initialPresentationClass = ExistingSpeciesOverviewPresentation.class)
public class ExistingSpeciesDelete extends BWSpeciesFunction {

	/**
	 * 
	 */
	private static final String NLSPECIESNAMELIST = "NLSPECIESNAMELIST"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final NameIdentifier NLSPECIESNAMELIST_I = NameIdentifier.get(NLSPECIESNAMELIST);

	/**
	 * 
	 */
	private boolean leave = false;

	/**
	 * @throws ASPLException
	 * 
	 */
	@SuppressWarnings("unchecked")
	private void processDeleteConfirmation ( ) throws ASPLException {

		
		try (DBConnection dbc = getDBConnection();) {
			LinkedList<DmlAssignmentCommand> commands = (LinkedList<DmlAssignmentCommand>) getFromFunctionContextMandatory("commands"); //$NON-NLS-1$
			if (((LogicalValue) getPresentationValue("UNMARKASPOSSIBLE")).getValue()) { //$NON-NLS-1$
				commands.addLast((DmlAssignmentCommand) getFromFunctionContextMandatory("UNMARKASPOSSIBLE")); //$NON-NLS-1$
			}
			DmlCommand.execDmlCommandAndEndTransaction(dbc.startTransaction(getUserData().getUserID(), true, "", new byte[] {}, TransactionMode.UPDATE), new DmlMultipleAssignmentCommand(commands)); //$NON-NLS-1$
			leave = true;
		} catch (ConnectionClosedException e) {
			log(e);
			throw new ASPLApplicationException(e);
		} catch (ErrorMessageException e) {
			log(e);
			throw new ASPLApplicationException(e);
		}

	}

	/**
	 * @throws ASPLException
	 * 
	 */
	private void processSelectionPresentation ( ) throws ASPLException {

		Relation r = getMatchingSpecies();

		if (r.size() == 1) {
			ExistingSpecies species;
			try {
				species = DBObjectFactory.getObjectCollectionIgnoringMissingSetters(r, ExistingSpecies.class).iterator().next();
			} catch (ConstructorMissingException e) {
				throw new ASPLApplicationException(e);
			} catch (ClassDoesNotImplementDBObjectException e) {
				throw new ASPLApplicationException(e);
			}
			setPresentation(new ExistingSpeciesDeleteConfirmPresentation());
			setPresentationValue(ATTRIBUTENAMES.SPECIESNAME, species.getSpeciesName());

			Relation nlNames = r.iterator().next().rvaValue(NLSPECIESNAMELIST_I);
			Locale locale = getUserLocale();
			setPresentationValue(NLSPECIESNAMELIST, ASPLListFactory.getASPLListValue(getPresentation().getPresentationField(NLSPECIESNAMELIST), nlNames, getUserData(), locale));

			LinkedList<DmlAssignmentCommand> commands = new LinkedList<DmlAssignmentCommand>();
			commands.add(species.dbDeleteCommand(RELVARNAMES.EXISTINGSPECIES));
			for (Tuple nlNameTuple : nlNames) {
				commands.add(new NLSpeciesName(species.getSpeciesName(), nlNameTuple.valueText(ATTRIBUTENAMES.LANGUAGE_I), nlNameTuple.valueText(ATTRIBUTENAMES.NLSPECIESNAME_I)).dbDeleteCommand(RELVARNAMES.NLSPECIESNAME));
			}

			saveInFunctionContext("commands", commands); //$NON-NLS-1$
			saveInFunctionContext("UNMARKASPOSSIBLE", species.dbUnAssertCommand(RELVARNAMES.POSSIBLESPECIES)); //$NON-NLS-1$
		} else {
			Locale locale = getUserLocale();
			setPresentationValue("SPECIESLIST", ASPLListFactory.getASPLListValue(getPresentation().getPresentationField("SPECIESLIST"), r, getUserData(), locale)); //$NON-NLS-1$ //$NON-NLS-2$
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doInitialLogic()
	 */
	public void doInitialLogic ( ) throws ASPLException {
		setDefaultLanguageFromUserContext();
		if (getPresentationValue(ATTRIBUTENAMES.SPECIESNAME) != null || getPresentationValue(ATTRIBUTENAMES.NLSPECIESNAME) != null || getPresentationValue("SPECIESNAMEPATTERN") != null || getPresentationValue("NLSPECIESNAMEPATTERN") != null) {//$NON-NLS-1$//$NON-NLS-2$
			processSelectionPresentation();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doPresentationLogic()
	 */
	public void doPresentationLogic ( ) throws ASPLException {
		if (getPresentation() instanceof ExistingSpeciesOverviewPresentation) {
			processSelectionPresentation();
		} else {
			processDeleteConfirmation();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doResumeLogic()
	 */
	public void doResumeLogic ( ) throws ASPLException {
		doPresentationLogic();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#getPresentationFunctionDescription()
	 */
	public String getPresentationFunctionDescription () {
		return I18N.getString(getUserLocale(), "ExistingSpeciesDelete.Text"); //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#leaveFunctionOnExit()
	 */
	public boolean leaveFunctionOnExit ( ) {
		return leave;
	}
}