package be.SIRAPRISE.birdwatcher;

import java.util.LinkedList;
import java.util.Locale;

import be.ASPL.presentationlayer.*;
import be.SIRAPRISE.client.*;
import be.SIRAPRISE.client.jsba.*;
import be.SIRAPRISE.messages.ErrorMessageException;
import be.SIRAPRISE.typeimplementations.Relation;
import be.SIRAPRISE.util.ISODateFormat;

/**
 * The function to produce an overview of defined attributes
 * 
 * @author Erwin
 */
@InitialPresentation(initialPresentationClass = WalksOverviewPresentation.class)
public class WalksDelete extends BWWalksFunction {

	/**
	 * 
	 */
	private boolean leave = false;

	/**
	 * @throws ASPLException
	 * 
	 */
	private void processDeleteConfirmation ( ) throws ASPLException {

		Walk walk = (Walk) getFromFunctionContextMandatory(RELVARNAMES.WALK);
		LinkedList<ServerSingleAssignmentCommand> commands = new LinkedList<ServerSingleAssignmentCommand>();
		String dateCondition = "EQ(" + ATTRIBUTENAMES.WALKDATE + ",DATE(" + ISODateFormat.formatISO(walk.getWalkDate()) + "))"; //$NON-NLS-1$//$NON-NLS-2$ //$NON-NLS-3$
		commands.addLast(walk.dbDeleteCommand(RELVARNAMES.WALK).getServerCommand());
		commands.addLast(new ServerDeleteCommand(RELVARNAMES.SPOTS, "RESTRICT(" + RELVARNAMES.SPOTS + "," + dateCondition + ")")); //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
		commands.addLast(new ServerDeleteCommand(RELVARNAMES.GSPOTS, "RESTRICT(" + RELVARNAMES.GSPOTS + "," + dateCondition + ")")); //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
		
		try (DBConnection dbc = getDBConnection();) {
			dbc.startTransaction(getUserData().getUserID(), true, "", new byte[] {}, TransactionMode.UPDATE).execServerCommandAndEndTransaction(new ServerMultipleAssignmentCommand(commands)); //$NON-NLS-1$
			leave = true;
		} catch (ConnectionClosedException e) {
			log(e);
			throw new ASPLApplicationException(e);
		} catch (ErrorMessageException e) {
			log(e);
			throw new ASPLApplicationException(e);
		}
	}

	/**
	 * @throws ASPLException
	 */
	private void processSelectionPresentation ( ) throws ASPLException {
		Relation r = getMatchingWalks();
		Locale locale = getUserLocale();
		if (r.size() == 1) {
			try {
				Walk walk = DBObjectFactory.getObjectCollectionIgnoringMissingSetters(r, Walk.class).iterator().next();

				DateValue wv_walkDate = new DateValue(walk.getWalkDate());
				PresentationValue wv_language = getPresentationValue(ATTRIBUTENAMES.LANGUAGE);
				setPresentation(new WalksDeleteConfirmPresentation());
				setPresentationValue(ATTRIBUTENAMES.WALKDATE, wv_walkDate);
				setPresentationValue(ATTRIBUTENAMES.LANGUAGE, wv_language);
				setPresentationValue("SPOTSLIST", ASPLListFactory.getASPLListValue(getPresentation().getPresentationField("SPOTSLIST"), getMatchingSpots(), getUserData(), locale)); //$NON-NLS-1$ //$NON-NLS-2$

				saveInFunctionContext(RELVARNAMES.WALK, walk);
			} catch (ConstructorMissingException e) {
				throw new RuntimeException(e);
			} catch (ClassDoesNotImplementDBObjectException e) {
				throw new RuntimeException(e);
			}
		} else {
			setPresentationValue("WALKSLIST", ASPLListFactory.getASPLListValue(getPresentation().getPresentationField("WALKSLIST"), r, getUserData(), locale)); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doInitialLogic()
	 */
	public void doInitialLogic ( ) throws ASPLException {
		setDefaultLanguageFromUserContext();

		if (getPresentationValue(ATTRIBUTENAMES.WALKDATE) != null || getPresentationValue("FROMDATE") != null || getPresentationValue("TODATE") != null || getPresentationValue(ATTRIBUTENAMES.SPECIESNAME) != null) { //$NON-NLS-1$ //$NON-NLS-2$
			processSelectionPresentation();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#doPresentationLogic()
	 */
	public void doPresentationLogic ( ) throws ASPLException {
		if (getPresentation().getName().equalsIgnoreCase("WalksOverview")) { //$NON-NLS-1$
			processSelectionPresentation();
		} else {
			processDeleteConfirmation();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#getPresentationFunctionDescription()
	 */
	public String getPresentationFunctionDescription () {
		return I18N.getString(getUserLocale(), "WalksDelete.Text"); //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.OnLinePresentationFunction#leaveFunctionOnExit()
	 */
	public boolean leaveFunctionOnExit ( ) {
		return leave;
	}
}