/**
 * 
 */
package be.SIRAPRISE.tests.officialtestset;

import static org.junit.Assert.fail;

import java.io.IOException;
import java.util.*;

import org.junit.*;

import be.SIRAPRISE.client.*;
import be.SIRAPRISE.client.NAMES.*;
import be.SIRAPRISE.client.jsba.*;
import be.SIRAPRISE.messages.CommunicationProtocolException;
import be.SIRAPRISE.messages.ErrorMessageException;
import be.SIRAPRISE.typeimplementations.TYPENAMES;
import be.SIRAPRISE.util.*;

/**
 * @author Erwin
 */
public final class EngineTransactionsLockingTest {

	/**
	 * 
	 */
	private static final String TXL_L1 = "TXL_L1"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String TXL_H = "TXL_H"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String TXL_H_R1 = TXL_H + "_R1"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String TXL_SPDB = "TXL.SPDB"; //$NON-NLS-1$

	/**
	 * Accepts commands from the console and runs them through the server
	 * 
	 * @param args
	 *            No arguments are used
	 */
	public static void main (String args[]) {
		final EngineTransactionsLockingTest engineTransactionsLockingTest = new EngineTransactionsLockingTest();
		engineTransactionsLockingTest.setProperties(MyCommandLineParameters.getCommandLineParameters(args, null, EngineTransactionsLockingTestProperties.getInstance()));
		engineTransactionsLockingTest.test();
	}

	/**
	 * 
	 */
	private String hostName;

	/**
	 * 
	 */
	private int monitorport;

	/**
	 * 
	 */
	private int port;

	/**
	 * Constructor for the JUnit framework
	 */
	public EngineTransactionsLockingTest ( ) {

	}

	/**
	 * @param properties
	 */
	private void setProperties (Properties properties) {
		hostName = properties.getProperty(DBConnectionProperties.HOST, DBConnectionProperties.HOSTDEFAULT);
		port = Integer.parseInt(properties.getProperty(DBConnectionProperties.PORT, DBConnectionProperties.PORTDEFAULT));
		monitorport = Integer.parseInt(properties.getProperty(DBConnectionProperties.MONITORPORT, DBConnectionProperties.MONITORPORTDEFAULT));
	}

	/**
	 * 
	 */
	@Before
	public void before ( ) {
		setProperties(EngineTransactionsLockingTestProperties.getInstance());
	}

	/**
	 * 
	 */
	@SuppressWarnings("resource")
	@Test
	public void test ( ) {

		// Get the connection
		OneShotDBConnection dbc1 = null;
		OneShotDBConnection dbc2 = null;
		OneShotDBConnection dbc3 = null;
		do {
			try {
				// get the DB connection
				// We request a longer idle time (10 sec) than usual because otherwise the socket read to get the commit on the update transacction will time out too soon.
				dbc1 = new OneShotDBConnection(new SiraPriseServer(hostName, port, monitorport), "", 10000, null); //$NON-NLS-1$
			} catch (DBException e1) {
				if (e1.getErrorCode() == 454) { // max connections exceeded
					try {
						Thread.sleep(200);
					} catch (InterruptedException e) {

					}
				} else {
					fail(e1.getClass().getName() + ' ' + e1.getMessage());
				}
			} catch (IOException e) {
				fail(e.getClass().getName() + ' ' + e.getMessage());
			} catch (CommunicationProtocolException e) {
				fail(e.getClass().getName() + ' ' + e.getMessage());
			}
		} while (dbc1 == null);

		do {
			try {
				// get the DB connection
				dbc2 = new OneShotDBConnection(new SiraPriseServer(hostName, port, monitorport), "", 10000, null); //$NON-NLS-1$
			} catch (DBException e1) {
				if (e1.getErrorCode() == 454) { // max connections exceeded
					try {
						Thread.sleep(200);
					} catch (InterruptedException e) {

					}
				} else {
					fail(e1.getClass().getName() + ' ' + e1.getMessage());
				}
			} catch (IOException e) {
				fail(e.getClass().getName() + ' ' + e.getMessage());
			} catch (CommunicationProtocolException e) {
				fail(e.getClass().getName() + ' ' + e.getMessage());
			}
		} while (dbc2 == null);

		do {
			try {
				// get the DB connection
				dbc3 = new OneShotDBConnection(new SiraPriseServer(hostName, port, monitorport), "", 10000, null); //$NON-NLS-1$
			} catch (DBException e1) {
				if (e1.getErrorCode() == 454) { // max connections exceeded
					try {
						Thread.sleep(200);
					} catch (InterruptedException e) {

					}
				} else {
					fail(e1.getClass().getName() + ' ' + e1.getMessage());
				}
			} catch (IOException e) {
				fail(e.getClass().getName() + ' ' + e.getMessage());
			} catch (CommunicationProtocolException e) {
				fail(e.getClass().getName() + ' ' + e.getMessage());
			}
		} while (dbc3 == null);

		// Start a transaction and start processing
		try {
			final Attribute a1 = new Attribute(TXL_L1, TYPENAMES.LONG, -1);
			final Relvar rh = new Relvar(TXL_H, "TXL_L1 is \u00A7TXL_l1\u00A7."); //$NON-NLS-1$
			final RelvarAttribute rh_l1 = new RelvarAttribute(TXL_H, TXL_L1);

			final DbmsFile f = new DbmsFile(TXL_SPDB, 8192);
			final HashedRecordSpace hrs1 = new HashedRecordSpace(TXL_SPDB, 1, 1668, 9, 15, 4);

			final RecordType rh1 = new RecordType(TXL_H, TXL_H_R1, 60, TXL_SPDB, 1);
			final RecordAttribute rh11 = new RecordAttribute(TXL_H, TXL_H_R1, 60, TXL_L1);
			final IndexAttribute ia1 = new IndexAttribute(TXL_H, TXL_H_R1, 10, TXL_L1);

			final DBTransaction ddlTransactionCreate = dbc1.startTransaction(true);
			DmlCommand.execDmlCommand(ddlTransactionCreate, a1.dbAssertCommand(RELVARNAMES.ATTRIBUTE));
			DmlCommand.execDmlCommand(ddlTransactionCreate, new DmlMultipleAssignmentCommand(rh.dbAssertCommand(RELVARNAMES.RELVAR), rh_l1.dbAssertCommand(RELVARNAMES.RELVARATTRIBUTE)));
			DmlCommand.execDmlCommand(ddlTransactionCreate, new DmlMultipleAssignmentCommand(f.dbAssertCommand(RELVARNAMES.DBMSFILE), hrs1.dbAssertCommand(RELVARNAMES.STORAGESPACE), hrs1.dbAssertCommand(RELVARNAMES.HASHEDRECORDSPACE)));
			DmlCommand.execDmlCommand(ddlTransactionCreate, new DmlMultipleAssignmentCommand(rh1.dbAssertCommand(RELVARNAMES.RECORDTYPE), rh11.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE), ia1.dbAssertCommand(RELVARNAMES.INDEXATTRIBUTE)));
			ddlTransactionCreate.end(true);

			// Do the actual testing

			final DBTransaction dmlTransaction1 = dbc1.startTransaction(false);
			long tm = System.currentTimeMillis();
			dmlTransaction1.execServerCommand(new ServerUnassertCommand(TXL_H, TXL_H));
			dmlTransaction1.commit();
			System.out.println("Clearance in " + (System.currentTimeMillis() - tm) + " mils."); //$NON-NLS-1$ //$NON-NLS-2$

			final DBTransaction dmlTransaction2 = dbc2.startTransaction(false, TransactionMode.READONLY, SnapShotIsolationMode.OFF);
			final DBTransaction dmlTransaction3 = dbc3.startTransaction(false, TransactionMode.READONLY, SnapShotIsolationMode.ON);
			// Do an insert
			dmlTransaction1.execServerCommand(new ServerAssertCommand(TXL_H, TXL_H + "(BODY(TUPLE(" + TXL_L1 + "(7))))")); //$NON-NLS-1$//$NON-NLS-2$

			// Query via transaction 2 ; must fail on lock
			try {
				fail("Transaction has not failed on lock wait timeout and returned a relation of size " + dmlTransaction2.execQuery(TXL_H).size() + "."); //$NON-NLS-1$ //$NON-NLS-2$
			} catch (ErrorMessageException e) {

			}

			// Query via transaction 3 ; must succeed and get an empty relation
			if (dmlTransaction3.execQuery(TXL_H).size() > 0) {
				fail("MVCC transaction did not return empty relation."); //$NON-NLS-1$
			}

			// commit the insert
			dmlTransaction1.commit();

			// Now repeat the query; now should work
			if (dmlTransaction2.execQuery(TXL_H).size() != 1) {
				fail("2PL transaction did not return singleton relation."); //$NON-NLS-1$
			}

			// Query via transaction 3 ; must succeed and still get an empty relation
			if (dmlTransaction3.execQuery(TXL_H).size() > 0) {
				fail("MVCC transaction did not return empty relation."); //$NON-NLS-1$
			}

			try {
				// must fail because transaction2 still has the read lock
				dmlTransaction1.execServerCommand(new ServerUnassertCommand(TXL_H, TXL_H));
				fail("Clearing relvar did not fail on obtaining an exclusive lock"); //$NON-NLS-1$
			} catch (ErrorMessageException e) {

			}

			// End the readonly transactions
			dmlTransaction2.end(true);
			dmlTransaction3.end(true);

			tm = System.currentTimeMillis();
			dmlTransaction1.execServerCommand(new ServerUnassertCommand(TXL_H, TXL_H));
			dmlTransaction1.commit();
			System.out.println("Clearance in " + (System.currentTimeMillis() - tm) + " mils."); //$NON-NLS-1$ //$NON-NLS-2$

			// end the transactions
			dmlTransaction1.end(true);

			// Start cleaning up the test case from the catalog
			final DBTransaction ddlTransactionDestroy = dbc1.startTransaction(true);
			DmlCommand.execDmlCommand(ddlTransactionDestroy, new DmlMultipleAssignmentCommand(rh1.dbUnAssertCommand(RELVARNAMES.RECORDTYPE), rh11.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE), ia1.dbUnAssertCommand(RELVARNAMES.INDEXATTRIBUTE)));
			DmlCommand.execDmlCommand(ddlTransactionDestroy, new DmlMultipleAssignmentCommand(f.dbUnAssertCommand(RELVARNAMES.DBMSFILE), hrs1.dbUnAssertCommand(RELVARNAMES.STORAGESPACE), hrs1.dbUnAssertCommand(RELVARNAMES.HASHEDRECORDSPACE)));
			DmlCommand.execDmlCommand(ddlTransactionDestroy, new DmlMultipleAssignmentCommand(rh_l1.dbUnAssertCommand(RELVARNAMES.RELVARATTRIBUTE), rh.dbUnAssertCommand(RELVARNAMES.RELVAR)));
			DmlCommand.execDmlCommand(ddlTransactionDestroy, new DmlMultipleAssignmentCommand(a1.dbUnAssertCommand(RELVARNAMES.ATTRIBUTE)));
			ddlTransactionDestroy.end(true);

		} catch (ConnectionClosedException e) {
			e.printStackTrace();
			fail(e.getClass().getName() + ' ' + e.getMessage());
		} catch (ErrorMessageException e) {
			e.printStackTrace();
			fail(e.getClass().getName() + ' ' + e.getMessage());
		} finally {
			dbc1.close();
			dbc2.close();
			dbc3.close();
		}
	}
}
