/**
 * 
 */
package be.SIRAPRISE.tests.officialtestset;

import static org.junit.Assert.fail;

import java.io.IOException;
import java.util.Properties;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.junit.*;

import be.SIRAPRISE.client.*;
import be.SIRAPRISE.client.NAMES.*;
import be.SIRAPRISE.client.jsba.*;
import be.SIRAPRISE.messages.CommunicationProtocolException;
import be.SIRAPRISE.messages.ErrorMessageException;
import be.SIRAPRISE.typeimplementations.DbmsLongImplementation;
import be.SIRAPRISE.typeimplementations.NameIdentifier;
import be.SIRAPRISE.typeimplementations.Relation;
import be.SIRAPRISE.typeimplementations.TYPENAMES;
import be.SIRAPRISE.typeimplementations.Tuple;
import be.SIRAPRISE.util.MyCommandLineParameters;
import be.SIRAPRISE.util.NotFoundException;

/**
 * @author Erwin
 * 
 */
public final class EngineVolumeUpdatingTest {

	/**
	 * 
	 */
	private static final NameIdentifier VUT_L1 = NameIdentifier.get("VUT_L1"); //$NON-NLS-1$

	/**
	 * 
	 */
	private static final NameIdentifier VUT_L2 = NameIdentifier.get("VUT_L2"); //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String VUT_H = "VUT_H"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String VUT_H_R1 = VUT_H + "_R1"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String VUT_H_R2 = VUT_H + "_R2"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String VUT_T = "VUT_T"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String VUT_T_R1 = VUT_T + "_R1"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String VUT_T_R2 = VUT_T + "_R2"; //$NON-NLS-1$

	/**
	 * 
	 */
	private static final String VUT_SPDB = "VUT.SPDB"; //$NON-NLS-1$

	/**
	 * Accepts commands from the console and runs them through the server
	 * 
	 * @param args
	 *            No arguments are used
	 */
	public static void main (String args[]) {
		final EngineVolumeUpdatingTest engineVolumeStressTest = new EngineVolumeUpdatingTest();
		engineVolumeStressTest.setProperties(MyCommandLineParameters.getCommandLineParameters(args, null, EngineVolumeUpdatingTestProperties.getInstance()));
		engineVolumeStressTest.test();
	}

	/**
	 * 
	 */
	private String hostName;

	/**
	 * 
	 */
	private int monitorport;

	/**
	 * 
	 */
	private int port;

	/**
	 * 
	 */
	private int totalBatchSize;

	/**
	 * 
	 */
	private int commitCounter;

	/**
	 * Constructor for the JUnit framework
	 */
	public EngineVolumeUpdatingTest ( ) {

	}

	/**
	 * @param properties
	 */
	private void setProperties (Properties properties) {
		hostName = properties.getProperty(DBConnectionProperties.HOST, DBConnectionProperties.HOSTDEFAULT);
		port = Integer.parseInt(properties.getProperty(DBConnectionProperties.PORT, DBConnectionProperties.PORTDEFAULT));
		monitorport = Integer.parseInt(properties.getProperty(DBConnectionProperties.MONITORPORT, DBConnectionProperties.MONITORPORTDEFAULT));
		totalBatchSize = Integer.parseInt(properties.getProperty(EngineVolumeSpeedTestProperties.TOTALBATCHSIZE, EngineVolumeSpeedTestProperties.TOTALBATCHSIZEDEFAULT));
		commitCounter = Integer.parseInt(properties.getProperty(EngineVolumeSpeedTestProperties.COMMITSIZE, EngineVolumeSpeedTestProperties.COMMITSIZEDEFAULT));
	}

	/**
	 * 
	 */
	@Before
	public void before ( ) {
		setProperties(EngineVolumeSpeedTestProperties.getInstance());
	}

	/**
	 * 
	 */
	@SuppressWarnings("resource")
	@Test
	public void test ( ) {

		// Get the connection
		OneShotDBConnection dbc = null;
		do {
			try {
				// get the DB connection
				dbc = new OneShotDBConnection(new SiraPriseServer(hostName, port, monitorport), "", null); //$NON-NLS-1$
			} catch (DBException e1) {
				if (e1.getErrorCode() == 454) { // max connections exceeded
					try {
						Thread.sleep(200);
					} catch (InterruptedException e) {

					}
				} else {
					fail(e1.getClass().getName() + ' ' + e1.getMessage());
				}
			} catch (IOException e) {
				fail(e.getClass().getName() + ' ' + e.getMessage());
			} catch (CommunicationProtocolException e) {
				fail(e.getClass().getName() + ' ' + e.getMessage());
			}
		} while (dbc == null);

		// Start a transaction and start processing
		try {
			final Attribute a1 = new Attribute(VUT_L1.getString(), TYPENAMES.LONG, -1);
			final Attribute a2 = new Attribute(VUT_L2.getString(), TYPENAMES.LONG, -1);
			final Relvar rh = new Relvar(VUT_H, "VUT_L1 is \u00A7VUT_l1\u00A7 and VUT_L2 is \u00A7VUT_l2\u00A7."); //$NON-NLS-1$
			final Relvar rt = new Relvar(VUT_T, "VUT_L1 is \u00A7VUT_l1\u00A7 and VUT_L2 is \u00A7VUT_l2\u00A7."); //$NON-NLS-1$
			final RelvarAttribute rh_l1 = new RelvarAttribute(VUT_H, VUT_L1.getString());
			final RelvarAttribute rh_l2 = new RelvarAttribute(VUT_H, VUT_L2.getString());
			final RelvarAttribute rt_l1 = new RelvarAttribute(VUT_T, VUT_L1.getString());
			final RelvarAttribute rt_l2 = new RelvarAttribute(VUT_T, VUT_L2.getString());

			final DbmsFile f = new DbmsFile(VUT_SPDB, 8192);
			final HashedRecordSpace hrs1 = new HashedRecordSpace(VUT_SPDB, 1, 1668, 9, 15, 4);
			final HashedRecordSpace hrs2 = new HashedRecordSpace(VUT_SPDB, 2, 1668, 9, 15, 4);
			final ClusteredRecordSpace crs1 = new ClusteredRecordSpace(VUT_SPDB, 11, 1668, 9);
			final ClusteredRecordSpace crs2 = new ClusteredRecordSpace(VUT_SPDB, 12, 1668, 9);
			final ClusteredRecordSpaceLocAttrs crs1la = new ClusteredRecordSpaceLocAttrs(VUT_SPDB, 11, 1, VUT_L1.getString());
			final ClusteredRecordSpaceLocAttrs crs2la = new ClusteredRecordSpaceLocAttrs(VUT_SPDB, 12, 1, VUT_L2.getString());

			final RecordType rh2 = new RecordType(VUT_H, VUT_H_R2, 60, VUT_SPDB, 1);
			final RecordType rh1 = new RecordType(VUT_H, VUT_H_R1, 60, VUT_SPDB, 2);
			final RecordType rt2 = new RecordType(VUT_T, VUT_T_R2, 60, VUT_SPDB, 12);
			final RecordType rt1 = new RecordType(VUT_T, VUT_T_R1, 60, VUT_SPDB, 11);

			final RecordAttribute rh11 = new RecordAttribute(VUT_H, VUT_H_R1, 60, VUT_L1.getString());
			final RecordAttribute rh12 = new RecordAttribute(VUT_H, VUT_H_R1, 120, VUT_L2.getString());
			final RecordAttribute rh21 = new RecordAttribute(VUT_H, VUT_H_R2, 60, VUT_L1.getString());
			final RecordAttribute rh22 = new RecordAttribute(VUT_H, VUT_H_R2, 120, VUT_L2.getString());
			final RecordAttribute rt11 = new RecordAttribute(VUT_T, VUT_T_R1, 60, VUT_L1.getString());
			final RecordAttribute rt12 = new RecordAttribute(VUT_T, VUT_T_R1, 120, VUT_L2.getString());
			final RecordAttribute rt21 = new RecordAttribute(VUT_T, VUT_T_R2, 60, VUT_L1.getString());
			final RecordAttribute rt22 = new RecordAttribute(VUT_T, VUT_T_R2, 120, VUT_L2.getString());

			final IndexAttribute ia1 = new IndexAttribute(VUT_H, VUT_H_R1, 10, VUT_L1.getString());
			final IndexAttribute ia2 = new IndexAttribute(VUT_H, VUT_H_R2, 10, VUT_L2.getString());

			DBTransaction ddlTransactionCreate = dbc.startTransaction(true);

			DmlAssertCommand ac1 = a1.dbAssertCommand(RELVARNAMES.ATTRIBUTE);
			DmlAssertCommand ac2 = a2.dbAssertCommand(RELVARNAMES.ATTRIBUTE);
			DmlCommand.execDmlCommand(ddlTransactionCreate, new DmlMultipleAssignmentCommand(ac1, ac2));

			DmlAssertCommand rc1 = rh.dbAssertCommand(RELVARNAMES.RELVAR);
			DmlAssertCommand rc2 = rt.dbAssertCommand(RELVARNAMES.RELVAR);
			DmlAssertCommand rac11 = rh_l1.dbAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlAssertCommand rac12 = rh_l2.dbAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlAssertCommand rac21 = rt_l1.dbAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlAssertCommand rac22 = rt_l2.dbAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlCommand.execDmlCommand(ddlTransactionCreate, new DmlMultipleAssignmentCommand(rc1, rc2, rac11, rac12, rac21, rac22));

			DmlAssertCommand fc = f.dbAssertCommand(RELVARNAMES.DBMSFILE);
			DmlAssertCommand hc0s = hrs1.dbAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlAssertCommand hc0h = hrs1.dbAssertCommand(RELVARNAMES.HASHEDRECORDSPACE);
			DmlAssertCommand hc1s = hrs2.dbAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlAssertCommand hc1h = hrs2.dbAssertCommand(RELVARNAMES.HASHEDRECORDSPACE);
			DmlAssertCommand cc0s = crs1.dbAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlAssertCommand cc0c = crs1.dbAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACE);
			DmlAssertCommand cc1s = crs2.dbAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlAssertCommand cc1c = crs2.dbAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACE);

			DmlAssertCommand ccl1 = crs1la.dbAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACELOCATTRS);
			DmlAssertCommand ccl2 = crs2la.dbAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACELOCATTRS);

			DmlCommand.execDmlCommand(ddlTransactionCreate, new DmlMultipleAssignmentCommand(fc, hc0s, hc0h, hc1s, hc1h, cc0s, cc0c, cc1s, cc1c, ccl1, ccl2));

			DmlAssertCommand crt0h = rh2.dbAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlAssertCommand crt0c = rh1.dbAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlAssertCommand crt1h = rt2.dbAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlAssertCommand crt1c = rt1.dbAssertCommand(RELVARNAMES.RECORDTYPE);

			DmlAssertCommand cra0h = rh21.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra0c = rh11.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra1h = rt21.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra1c = rt11.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra0h2 = rh22.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra0c2 = rh12.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra1h2 = rt22.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlAssertCommand cra1c2 = rt12.dbAssertCommand(RELVARNAMES.RECORDATTRIBUTE);

			DmlAssertCommand cia1 = ia1.dbAssertCommand(RELVARNAMES.INDEXATTRIBUTE);
			DmlAssertCommand cia2 = ia2.dbAssertCommand(RELVARNAMES.INDEXATTRIBUTE);

			DmlCommand.execDmlCommand(ddlTransactionCreate, new DmlMultipleAssignmentCommand(crt0h, crt0c, crt1h, crt1c, cra0h, cra0c, cra1h, cra1c, cra0h2, cra0c2, cra1h2, cra1c2, cia1, cia2));
			ddlTransactionCreate.end(true);

			// Do the actual testing

			DBTransaction dmlTransaction = dbc.startTransaction(false);
			long tm;

			tm = System.currentTimeMillis();
			dmlTransaction.execServerCommand(ServerCommand.getCommandObjectFrom("CMD(UNASSERT " + VUT_H + "," + VUT_H + ")CMD(UNASSERT " + VUT_T + "," + VUT_T + ")")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
			dmlTransaction.commit();
			System.out.println("Clearance in " + (System.currentTimeMillis() - tm) + " mils."); //$NON-NLS-1$ //$NON-NLS-2$
			tm = System.currentTimeMillis();

			long stop = tm + 3600000;
			int moreDeletesThanInsertsCount = 0;
			do {
				int updateCounter = 0;
				int testedDeletes = 0;
				int testedInserts = 0;
				while (updateCounter++ < commitCounter) {
					int flipCoin;
					if (moreDeletesThanInsertsCount <= 20) {
						flipCoin = (int) (totalBatchSize * Math.random());
					} else {
						// There have been twenty cases where more deletes were generated than inserts, so now we have tested "inserts exclusively", and also "inserts mixed with deletes", what's left to be done is forcing the test of "delete exclusively"
						flipCoin = -1;
					}
					// flipCoin in range 0-TOTALBATCHSIZE
					// if that is smaller than current size of the local copy, then we delete, else we insert
					// That way, the probability of a delete increases with map size, with absolute certainty reached at the localCopy size of TOTALBATCHSIZE,
					// and the probability of an insert increases with decreasing map size, with absolute certainty for insert reached at localCopy size 0.
					if (flipCoin < localCopy.size()) {
						if (localCopy.size() > 0) {
							testedDeletes++;
							testSomeDelete(dmlTransaction);
						}
					} else {
						testedInserts++;
						testSomeInsert(dmlTransaction);
					}
				}
				System.out.println("Performed " + testedInserts + " insert-tests and " + testedDeletes + " delete-tests in " + (0 - tm + (tm = System.currentTimeMillis())) + " --- " + (20 - moreDeletesThanInsertsCount)); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
				dmlTransaction.commit();
				System.out.println("Commit in " + (0 - tm + (tm = System.currentTimeMillis()))); //$NON-NLS-1$
				if (testedInserts < testedDeletes) {
					moreDeletesThanInsertsCount++;
				}
			} while (tm < stop && localCopy.size() > 0);

			tm = System.currentTimeMillis();
			dmlTransaction.execServerCommand(new ServerMultipleAssignmentCommand(new ServerUnassertCommand(VUT_H, VUT_H), new ServerUnassertCommand(VUT_T, VUT_T)));
			System.out.println("Cleanup in " + (System.currentTimeMillis() - tm) + " mils."); //$NON-NLS-1$ //$NON-NLS-2$
			tm = System.currentTimeMillis();
			dmlTransaction.end(true);
			System.out.println("commit in " + (System.currentTimeMillis() - tm) + " mils."); //$NON-NLS-1$ //$NON-NLS-2$

			// Start cleaning up the test case from the catalog

			DmlUnassertCommand drt0h = rh2.dbUnAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlUnassertCommand drt0c = rh1.dbUnAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlUnassertCommand drt1h = rt2.dbUnAssertCommand(RELVARNAMES.RECORDTYPE);
			DmlUnassertCommand drt1c = rt1.dbUnAssertCommand(RELVARNAMES.RECORDTYPE);

			DmlUnassertCommand dra0h = rh21.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra0c = rh11.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra1h = rt21.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra1c = rt11.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra0h2 = rh22.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra0c2 = rh12.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra1h2 = rt22.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);
			DmlUnassertCommand dra1c2 = rt12.dbUnAssertCommand(RELVARNAMES.RECORDATTRIBUTE);

			DmlUnassertCommand dia1 = ia1.dbUnAssertCommand(RELVARNAMES.INDEXATTRIBUTE);
			DmlUnassertCommand dia2 = ia2.dbUnAssertCommand(RELVARNAMES.INDEXATTRIBUTE);

			DBTransaction ddlTransactionDestroy = dbc.startTransaction(true);
			DmlCommand.execDmlCommand(ddlTransactionDestroy, new DmlMultipleAssignmentCommand(drt0h, drt0c, drt1h, drt1c, dra0h, dra0c, dra1h, dra1c, dra0h2, dra0c2, dra1h2, dra1c2, dia1, dia2));

			DmlUnassertCommand dcl1 = crs1la.dbUnAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACELOCATTRS);
			DmlUnassertCommand dcl2 = crs2la.dbUnAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACELOCATTRS);

			DmlUnassertCommand fd = f.dbUnAssertCommand(RELVARNAMES.DBMSFILE);
			DmlUnassertCommand hd0s = hrs1.dbUnAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlUnassertCommand hd0h = hrs1.dbUnAssertCommand(RELVARNAMES.HASHEDRECORDSPACE);
			DmlUnassertCommand hd1s = hrs2.dbUnAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlUnassertCommand hd1h = hrs2.dbUnAssertCommand(RELVARNAMES.HASHEDRECORDSPACE);
			DmlUnassertCommand dc0s = crs1.dbUnAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlUnassertCommand dc0c = crs1.dbUnAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACE);
			DmlUnassertCommand dc1s = crs2.dbUnAssertCommand(RELVARNAMES.STORAGESPACE);
			DmlUnassertCommand dc1c = crs2.dbUnAssertCommand(RELVARNAMES.CLUSTEREDRECORDSPACE);

			DmlCommand.execDmlCommand(ddlTransactionDestroy, new DmlMultipleAssignmentCommand(fd, hd0s, hd0h, hd1s, hd1h, dc0s, dc0c, dc1s, dc1c, dcl1, dcl2));

			DmlUnassertCommand rad1 = rh_l1.dbUnAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlUnassertCommand rad2 = rh_l2.dbUnAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlUnassertCommand rad3 = rt_l1.dbUnAssertCommand(RELVARNAMES.RELVARATTRIBUTE);
			DmlUnassertCommand rad4 = rt_l2.dbUnAssertCommand(RELVARNAMES.RELVARATTRIBUTE);

			DmlUnassertCommand rd1 = rh.dbUnAssertCommand(RELVARNAMES.RELVAR);
			DmlUnassertCommand rd2 = rt.dbUnAssertCommand(RELVARNAMES.RELVAR);

			DmlCommand.execDmlCommand(ddlTransactionDestroy, new DmlMultipleAssignmentCommand(rad1, rad2, rad3, rad4, rd1, rd2));

			DmlUnassertCommand ad1 = a1.dbUnAssertCommand(RELVARNAMES.ATTRIBUTE);
			DmlUnassertCommand ad2 = a2.dbUnAssertCommand(RELVARNAMES.ATTRIBUTE);
			DmlCommand.execDmlCommand(ddlTransactionDestroy, new DmlMultipleAssignmentCommand(ad1, ad2));

			ddlTransactionDestroy.end(true);

		} catch (ConnectionClosedException e) {
			fail(e.getClass().getName() + ' ' + e.getMessage());
		} catch (ErrorMessageException e) {
			fail(e.getClass().getName() + ' ' + e.getMessage());
		} catch (NotFoundException e) {
			fail(e.getClass().getName() + ' ' + e.getMessage());
		} catch (IllegalServerCommandException e) {
			fail(e.getClass().getName() + ' ' + e.getMessage());
		} finally {
			dbc.close();
		}
	}

	/**
	 * Comment for <code>localCopy</code>
	 */
	private TreeMap<Long, Long> localCopy = new TreeMap<Long, Long>();

	/**
	 * @param dmlTransaction
	 * @throws ConnectionClosedException
	 * @throws ErrorMessageException
	 */
	private void testSomeDelete (DBTransaction dmlTransaction) throws ConnectionClosedException, ErrorMessageException {
		Long Long1;
		long long1;
		SortedMap<Long, Long> subMap;
		do {
			// Pick a random value to produce a submap
			long1 = (long) (Long.MAX_VALUE * Math.random());
			Long1 = new Long(long1);
			subMap = localCopy.subMap(Long1, new Long(Long.MAX_VALUE));
		} while (subMap.size() < 1);

		Entry<Long, Long> me = subMap.entrySet().iterator().next();
		Long1 = me.getKey();
		long1 = (Long1).longValue();
		long long2 = (me.getValue()).longValue();

		// Maintain the localCopy
		localCopy.remove(Long1);

		// Prepare and execute the DELETE commands
		try {
			dmlTransaction.execServerCommand(ServerCommand.getCommandObjectFrom(new StringBuffer(1024).append("CMD(DELETE " + VUT_H + "," + VUT_H + "(TUPLE(" + VUT_L1 + "(" + long1 + ")" + VUT_L2 + "(" + long2 + "))))").append(new StringBuffer(1024).append("CMD(DELETE " + VUT_T + "," + VUT_T + "(TUPLE(" + VUT_L1 + "(" + long1 + ")" + VUT_L2 + "(" + long2 + "))))")).toString())); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$ //$NON-NLS-8$ //$NON-NLS-9$ //$NON-NLS-10$ //$NON-NLS-11$ //$NON-NLS-12$ //$NON-NLS-13$ //$NON-NLS-14$
		} catch (NotFoundException | IllegalServerCommandException e) {
			fail(e.getClass().getName() + '-' + e.getMessage());
		}

		Relation rh1 = dmlTransaction.execQuery("RESTRICT(" + VUT_H + ",EQ(" + VUT_L1 + ",LONG(" + long1 + ")))"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		Relation rt1 = dmlTransaction.execQuery("RESTRICT(" + VUT_T + ",EQ(" + VUT_L1 + ",LONG(" + long1 + ")))"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		Relation rh2 = dmlTransaction.execQuery("RESTRICT(" + VUT_H + ",EQ(" + VUT_L2 + ",LONG(" + long2 + ")))"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		Relation rt2 = dmlTransaction.execQuery("RESTRICT(" + VUT_T + ",EQ(" + VUT_L2 + ",LONG(" + long2 + ")))"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$

		if (rh1.size() != 0) {
			fail("Hash query 1 returned tuple after delete " + rh1.size()); //$NON-NLS-1$
		}
		if (rt1.size() != 0) {
			fail("Tree query 1 returned tuple after delete " + rt1.size()); //$NON-NLS-1$
		}
		if (rh2.size() != 0) {
			fail("Hash query 2 returned tuple after delete " + rh2.size()); //$NON-NLS-1$
		}
		if (rt2.size() != 0) {
			fail("Tree query 2 returned tuple after delete " + rt2.size()); //$NON-NLS-1$
		}
	}

	/**
	 * @param dmlTransaction
	 * @throws ConnectionClosedException
	 * @throws ErrorMessageException
	 */
	private void testSomeInsert (DBTransaction dmlTransaction) throws ConnectionClosedException, ErrorMessageException {
		long long1;
		long long2;
		Long Long1;
		Long Long2;
		do {
			long1 = (long) (Long.MAX_VALUE * Math.random());
			Long1 = new Long(long1);
			long2 = (long) (Long.MAX_VALUE * Math.random());
			Long2 = new Long(long2);
		} while (localCopy.containsKey(Long1));

		// Put in the localCopy
		localCopy.put(Long1, Long2);

		// Prepare and execute the ADD commands
		try {
			dmlTransaction.execServerCommand(ServerCommand.getCommandObjectFrom(new StringBuffer(1024).append("CMD(ADD " + VUT_H + "," + VUT_H + "(TUPLE(" + VUT_L1 + "(" + long1 + ")" + VUT_L2 + "(" + long2 + "))))").append(new StringBuffer(1024).append("CMD(ADD " + VUT_T + "," + VUT_T + "(TUPLE(" + VUT_L1 + "(" + long1 + ")" + VUT_L2 + "(" + long2 + "))))")).toString())); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$ //$NON-NLS-8$ //$NON-NLS-9$ //$NON-NLS-10$ //$NON-NLS-11$ //$NON-NLS-12$ //$NON-NLS-13$ //$NON-NLS-14$
		} catch (NotFoundException | IllegalServerCommandException e) {
			fail(e.getClass().getName() + '-' + e.getMessage());
		}

		// Immediately after the insert, all three should contain the inserted tuple
		Relation rh1 = dmlTransaction.execQuery("RESTRICT(" + VUT_H + ",EQ(" + VUT_L1 + ",LONG(" + long1 + ")))"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		Relation rt1 = dmlTransaction.execQuery("RESTRICT(" + VUT_T + ",EQ(" + VUT_L1 + ",LONG(" + long1 + ")))"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		Relation rh2 = dmlTransaction.execQuery("RESTRICT(" + VUT_H + ",EQ(" + VUT_L2 + ",LONG(" + long2 + ")))"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		Relation rt2 = dmlTransaction.execQuery("RESTRICT(" + VUT_T + ",EQ(" + VUT_L2 + ",LONG(" + long2 + ")))"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$

		if (rh1.size() != 1) {
			fail("Hash query 1 is non-singleton : " + rh1.size()); //$NON-NLS-1$
		}
		Tuple th1 = rh1.iterator().next();
		long h11 = DbmsLongImplementation.getJavaLong(th1.valueBuffer(VUT_L1));
		if (h11 != long1) {
			fail("Hash query 1 bad long1 value " + h11 + "(" + long1 + ")"); //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
		}
		long h12 = DbmsLongImplementation.getJavaLong(th1.valueBuffer(VUT_L2));
		if (h12 != long2) {
			fail("Hash query 1 bad long2 value " + h12 + "(" + long2 + ")"); //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
		}

		if (rt1.size() != 1) {
			fail("Tree query 1 size is non-singleton " + rt1.size()); //$NON-NLS-1$
		}
		Tuple tt1 = rt1.iterator().next();
		long t11 = DbmsLongImplementation.getJavaLong(tt1.valueBuffer(VUT_L1));
		if (t11 != long1) {
			fail("Tree query 1 bad long1 value " + t11 + "(" + long1 + ")"); //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
		}
		long t12 = DbmsLongImplementation.getJavaLong(tt1.valueBuffer(VUT_L2));
		if (t12 != long2) {
			fail("Tree Query 1 bad long2 value " + t12 + "(" + long2 + ")"); //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
		}

		if (rh2.size() < 1) {
			fail("Hash query 2 does not contain tuple for L2 " + long2); //$NON-NLS-1$
		}
		if (rh2.size() == 1) {
			Tuple th2 = rh2.iterator().next();
			long h21 = DbmsLongImplementation.getJavaLong(th2.valueBuffer(VUT_L1));
			if (h21 != long1) {
				fail("Hash query 2 bad long1 value " + h21 + "(" + long1 + ")"); //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
			}
			long h22 = DbmsLongImplementation.getJavaLong(th2.valueBuffer(VUT_L2));
			if (h22 != long2) {
				fail("Hash query 2 bad long2 value " + h22 + "(" + long2 + ")"); //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
			}
		} else {
			System.out.println("duplicate long2 value - unlikely but possible after insert"); //$NON-NLS-1$
		}

		if (rt2.size() < 1) {
			fail("Tree query 2 does not contain tuple for L2 " + long2); //$NON-NLS-1$
		}
		if (rt2.size() == 1) {
			Tuple tt2 = rt2.iterator().next();
			long t21 = DbmsLongImplementation.getJavaLong(tt2.valueBuffer(VUT_L1));
			if (t21 != long1) {
				fail("Tree query 2 bad long1 value " + t21 + "(" + long1 + ")"); //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
			}
			long t22 = DbmsLongImplementation.getJavaLong(tt2.valueBuffer(VUT_L2));
			if (t22 != long2) {
				fail("Tree query 2 bad long2 value " + t22 + "(" + long2 + ")"); //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$
			}
		} else {
			System.out.println("duplicate long2 value - unlikely but possible after insert"); //$NON-NLS-1$
		}
	}
}
